import org.jetbrains.kotlin.gradle.dsl.KotlinJsCompile
import org.jetbrains.kotlin.gradle.dsl.KotlinMultiplatformExtension
import org.jetbrains.kotlin.gradle.tasks.KotlinCompile

buildscript {
    repositories {
        mavenCentral()
        gradlePluginPortal()
    }

    dependencies {
        classpath(Plugins.kotlin)
        classpath(Plugins.kotlinSerialization)
    }
}

group = "org.example"
version = "0.1.0"

subprojects {
    repositories {
        google {
            content {
                includeGroupByRegex("^android\\..+")
                includeGroupByRegex("^androidx\\..+")
                includeGroupByRegex("^com\\.android\\.?.*\$")
                includeGroupByRegex("^com\\.google\\.?.*\$")
                includeGroupByRegex("^io\\.fabric\\.?.*\$")
                includeGroupByRegex("^com\\.crashlytics\\.?.*\$")
            }
        }
        maven {
            url = uri("https://kotlin.bintray.com/kotlinx")
            content {
                includeGroup("org.jetbrains.kotlinx")
            }
        }
        maven {
            url = uri("https://kotlin.bintray.com/ktor")
            content {
                includeGroup("io.ktor")
            }
        }
        mavenCentral()
    }

    tasks.withType<KotlinCompile> {
        kotlinOptions {
            jvmTarget = "1.8"
        }
    }

    tasks.withType<KotlinJsCompile> {
        kotlinOptions {
            sourceMap = true
        }
    }

    plugins.withId("org.jetbrains.kotlin.multiplatform") {
        configure<KotlinMultiplatformExtension> {
            sourceSets.configureEach {
                languageSettings.apply {
                    useExperimentalAnnotation("kotlin.ExperimentalStdlibApi")
                    useExperimentalAnnotation("kotlinx.coroutines.FlowPreview")
                    useExperimentalAnnotation("kotlinx.coroutines.ExperimentalCoroutinesApi")
                }
            }
        }
    }
}

tasks.wrapper {
    distributionType = Wrapper.DistributionType.ALL
}
