// 1 - For invalidating the cache

object AndroidConfig {
    const val compileSdkVersion = 29
    const val targetSdkVersion = 29
    const val minSdkVersion = 21
}

object Versions {
    const val androidx = "1.1.0"
    const val atomicfu = "0.14.4"
    const val kotlin = "1.4.0"
    const val kotlinxCoroutines = "1.3.9"
    const val kotlinxSerialization = "0.20.0"
    const val ktor = "1.3.2"
    const val kotlinHtml = "0.7.24"
    const val tornadofx = "1.7.20"
}

object Plugins {
    val android = "com.android.tools.build:gradle:3.6.0"
    val kotlin = "org.jetbrains.kotlin:kotlin-gradle-plugin:${Versions.kotlin}"
    val kotlinSerialization = "org.jetbrains.kotlin:kotlin-serialization:${Versions.kotlin}"
    val atomicfu = "org.jetbrains.kotlinx:atomicfu-gradle-plugin:${Versions.atomicfu}"
    val jnaerator_gradle = "org.anarres.gradle:gradle-jnaerator-plugin:1.0.0"
}

object Deps {
    val android = AndroidDeps()
    val androidx = AndroidXDeps()
    val jnaerator = "com.nativelibs4java:jnaerator:0.11"
    val kotlin = KotlinDeps()
    val kotlinHtml = KotlinHtmlDeps()
    val kotlinx = KotlinXDeps()
    val ktor = KtorDeps()
    val okio = "com.squareup.okio:okio:2.8.0"
    val okhttp = "com.squareup.okhttp3:okhttp:4.8.1"
    val tornadofx = "no.tornado:tornadofx:${Versions.tornadofx}"
}

class AndroidDeps internal constructor() {
    val robolectric = "org.robolectric:robolectric:4.3"
}

class AndroidXDeps internal constructor() {
    val appcompat = "androidx.appcompat:appcompat:${Versions.androidx}"
    val annotation = "androidx.annotation:annotation:${Versions.androidx}"
    val browser = "androidx.browser:browser:1.2.0"
}

class KotlinDeps internal constructor() {
    val stdlib = KotlinStdLibDeps()
    val test = KotlinTestDeps()
}

class KotlinHtmlDeps internal constructor() {
    val common = "dev.scottpierce.kotlin-html:kotlin-html-writer:${Versions.kotlinHtml}"
}

class KotlinStdLibDeps internal constructor() {
    val common = "org.jetbrains.kotlin:kotlin-stdlib-common:${Versions.kotlin}"
    val jvm = "org.jetbrains.kotlin:kotlin-stdlib-jdk8:${Versions.kotlin}"
    val js = "org.jetbrains.kotlin:kotlin-stdlib-js:${Versions.kotlin}"
}

class KotlinTestDeps internal constructor() {
    val common: List<String> = listOf(
        "org.jetbrains.kotlin:kotlin-test-common:${Versions.kotlin}",
        "org.jetbrains.kotlin:kotlin-test-annotations-common:${Versions.kotlin}"
    )
    val jvm = "org.jetbrains.kotlin:kotlin-test-junit:${Versions.kotlin}"
    val js = "org.jetbrains.kotlin:kotlin-test-js:${Versions.kotlin}"
}

class KotlinXDeps internal constructor() {
    val atomicfu = KotlinXAtomicFuDeps()
    val coroutines = KotlinXCoroutinesDeps()
    val nodejs = "org.jetbrains.kotlinx:kotlinx-nodejs:0.0.7"
    val serialization = KotlinXSerializationDeps()
}

class KotlinXCoroutinesDeps internal constructor() {
    val android = "org.jetbrains.kotlinx:kotlinx-coroutines-android:${Versions.kotlinxCoroutines}"
    val common = "org.jetbrains.kotlinx:kotlinx-coroutines-core:${Versions.kotlinxCoroutines}"
    val javafx = "org.jetbrains.kotlinx:kotlinx-coroutines-javafx:${Versions.kotlinxCoroutines}"
    val jvm = "org.jetbrains.kotlinx:kotlinx-coroutines-core-jvm:${Versions.kotlinxCoroutines}"
    val js = "org.jetbrains.kotlinx:kotlinx-coroutines-core-js:${Versions.kotlinxCoroutines}"
}

class KotlinXAtomicFuDeps internal constructor() {
    val common = "org.jetbrains.kotlinx:atomicfu-common:${Versions.atomicfu}"
    val jvm = "org.jetbrains.kotlinx:atomicfu:${Versions.atomicfu}"
}

class KotlinXSerializationDeps internal constructor() {
    val common = "org.jetbrains.kotlinx:kotlinx-serialization-runtime-common:${Versions.kotlinxSerialization}"
    val jvm = "org.jetbrains.kotlinx:kotlinx-serialization-runtime:${Versions.kotlinxSerialization}"
    val native = "org.jetbrains.kotlinx:kotlinx-serialization-runtime-native${Versions.kotlinxSerialization}"
    val js = "org.jetbrains.kotlinx:kotlinx-serialization-runtime-js:${Versions.kotlinxSerialization}"
}

class KtorDeps internal constructor() {
    val client = KtorClientDeps()
    val feature = KtorFeatureDeps()
}

class KtorClientDeps internal constructor() {
    val common = "io.ktor:ktor-client:${Versions.ktor}"
    val jvm = "io.ktor:ktor-client-okhttp:${Versions.ktor}"
    val ios = "io.ktor:ktor-client-ios:${Versions.ktor}"
}

class KtorFeatureDeps internal constructor() {
    val json = KtorFeatureJsonDeps()
    val serialization = KtorFeatureSeralizationDeps()
    val logging = KtorFeatureLoggingDeps()
    val auth = KtorFeatureAuthDeps()
}

class KtorFeatureJsonDeps internal constructor() {
    val jvm = "io.ktor:ktor-client-json-jvm:${Versions.ktor}"
    val ios = "io.ktor:ktor-client-json-ios:${Versions.ktor}"
    val common = "io.ktor:ktor-client-json:${Versions.ktor}"
}

class KtorFeatureSeralizationDeps internal constructor() {
    val common = "io.ktor:ktor-client-serialization:${Versions.ktor}"
    val jvm = "io.ktor:ktor-client-serialization-jvm:${Versions.ktor}"
    val native = "io.ktor:ktor-client-serialization-native:${Versions.ktor}"
}

class KtorFeatureLoggingDeps internal constructor() {
    val common = "io.ktor:ktor-client-logging:${Versions.ktor}"
    val jvm = "io.ktor:ktor-client-logging-jvm:${Versions.ktor}"
    val native = "io.ktor:ktor-client-logging-native:${Versions.ktor}"
}

class KtorFeatureAuthDeps internal constructor() {
    val common = "io.ktor:ktor-client-auth:${Versions.ktor}"
    val jvm = "io.ktor:ktor-client-auth-jvm:${Versions.ktor}"
    val native = "io.ktor:ktor-client-auth-native:${Versions.ktor}"
}
