package dev.scottpierce.mobiletest.repository

import dev.scottpierce.mobiletest.repository.data.DataSource
import kotlinx.coroutines.async
import kotlinx.coroutines.awaitAll

class DeviceRepository internal constructor(
    private val dataSource: DataSource
) : Repository() {
    suspend fun findDevices(): List<Device> = withRepositoryContext {
        println("FindDevices Map")
        dataSource.getConnectedDevices().map { device ->
            async {
                val data = dataSource.getDeviceInfo(device.id)
                println("getDeviceInfo Returned")
                Device(
                    id = device.id,
                    connectionType = device.type,
                    data = data
                )
            }
        }.awaitAll()
    }
}

data class Device(
    val id: String,
    val connectionType: String,
    val data: Map<String, String>
)
