package dev.scottpierce.mobiletest.repository

import dev.scottpierce.mobiletest.repository.data.DataSource
import kotlinx.coroutines.flow.*

class EventRepository internal constructor(
    private val dataSource: DataSource
) {
    suspend fun streamEvents(id: String): Flow<DeviceEvent> {
        return dataSource.streamSystemLog(id)
            .filter { it.isNotBlank() }
            .map { line ->
                when {
                    line.matches(DeviceEvent.VolumeUp.LINE_REGEX) -> DeviceEvent.VolumeUp
                    line.matches(DeviceEvent.VolumeDown.LINE_REGEX) -> DeviceEvent.VolumeDown
                    else -> null
                }
            }
            .filterNotNull()
    }
}

sealed class DeviceEvent {
    object VolumeUp : DeviceEvent() {
        /**
         * Aug 17 21:50:38 SpringBoard[49] <Notice>: VolumeControl -- volume increment up:1
         */
        val LINE_REGEX = Regex("^.+VolumeControl -- volume increment up:(\\d?)\$")
    }

    object VolumeDown : DeviceEvent() {
        /**
         * Aug 17 21:50:41 SpringBoard[49] <Notice>: VolumeControl -- volume increment down:1
         */
        val LINE_REGEX = Regex("^.+VolumeControl -- volume increment down:(\\d?)\$")
    }
}
