package dev.scottpierce.mobiletest.repository.data

import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.toList

class CommandLineDataSource : DataSource {
    companion object {
        private val CONNECTED_DEVICES_REGEX = Regex("(\\S+) \\((\\S+)\\)")
    }

    override suspend fun getConnectedDevices(): List<ConnectedDevice> {
        val output = CommandRunner.run("idevice_id")
            .toList()

        println("DataSource: $output")

        return output.map {
            val result = CONNECTED_DEVICES_REGEX.matchEntire(it)!!

            ConnectedDevice(
                id = result.groupValues[1],
                type = result.groupValues[2]
            )
        }
    }

    override suspend fun getDeviceInfo(id: String): Map<String, String> {
        val output = CommandRunner.run("ideviceinfo -u $id")
            .toList()

        return buildMap {
            for (line in output) {
                val split = line.split(':', limit = 2)

                if (split.size == 2) {
                    val key = split[0]
                    val value = split[1].trim()

                    if (key[0].isWhitespace()) {
                        // NonVolatileRAM:
                        //  auto-boot: dHJ1ZQ==
                        //  backlight-level: MTY4Mw==
                        //  boot-args:
                        //  oblit-begins: MjAxNC0wOS0wNSAxNDo0NTo0OSBHTVQtMDc6MDA6IE9ibGl0VHlwZTogT2JsaXRlcmF0ZURhdGFQYXJ0aXRpb24uIFJlYXNvbjogU0JSZXNldE1vZGU9Mw==
                        //  obliteration: MjAxNC0wOS0wNSAxNDo0NjowNSBHTVQtMDc6MDA6IGhhbmRsZV9tZXNzYWdlOiBBbmQgeW91IHdpbGwga25vdyBteSBuYW1lIGlzIHRoZSBMb3JkIHdoZW4gSSBsYXkgbXkgdmVuZ2VhbmNlIHVwb24gdGhlZS4=

                        // Skip this line as it's a sub item
                        continue
                    } else if (value.isBlank()) {
                        // No result for the value, so skip recording value
                        continue
                    } else {
                        put(key, value)
                    }
                } else {
                    // TODO Turn this into a log
                    throw RuntimeException("Unexpected line encountered: '$line'")
                }
            }
        }
    }

    override suspend fun streamSystemLog(id: String): Flow<String> {
        return CommandRunner.run("idevicesyslog -u $id")
    }
}

