@file:Suppress("UNCHECKED_CAST_TO_EXTERNAL_INTERFACE")

package dev.scottpierce.mobiletest.repository.data

import child_process.exec
import kotlinx.coroutines.*
import kotlinx.coroutines.channels.produce
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.receiveAsFlow

internal actual object CommandRunner {
    actual fun run(command: String): Flow<String> {
        return GlobalScope.produce {
            val process = exec(command)

            process.stdout!!.apply {
                setEncoding("utf8")

                on("data") { data: Any ->
                    val message = data.toString()
                    println("Data: $message")
                    launch {
                        send(message)
                    }
                }

                on("end") { data: Any? ->
                    launch {
                        close()
                    }
                }
            }

            process.stderr!!.on("data") { data: Any ->
                val message = data.toString()
                println("Error: $message")
                launch {
                    send(message)
                }
            }

            var processIsExiting = false
            process.on("exit") { _ ->
                processIsExiting = true
                println("Process On Exit")
                launch {
                    close()
                }
            }

            invokeOnClose {
                println("Invoke on Close")
                if (!processIsExiting) {
//                    process.kill()
                }
            }
        }.receiveAsFlow()
    }
}
