package dev.scottpierce.mobiletest.repository.data

import kotlinx.coroutines.CoroutineDispatcher
import kotlinx.coroutines.GlobalScope
import kotlinx.coroutines.asCoroutineDispatcher
import kotlinx.coroutines.channels.produce
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.receiveAsFlow
import okio.buffer
import okio.source
import java.util.concurrent.Executors

internal actual object CommandRunner {
    private val dispatcher: CoroutineDispatcher = Executors.newCachedThreadPool().asCoroutineDispatcher()

    actual fun run(command: String): Flow<String> {
        val process = ProcessBuilder(command.split(' ')).apply {
            redirectErrorStream(true)
        }.start()

        return GlobalScope.produce(dispatcher) {
            try {
                process.inputStream.source().use { processSource ->
                    processSource.buffer().use { bufferedProcessSource ->
                        while (true) {
                            @Suppress("BlockingMethodInNonBlockingContext")
                            val line = bufferedProcessSource.readUtf8Line() ?: break
                            send(line)
                        }
                    }
                }
            } catch(e: Exception) {
                e.printStackTrace()
                close(e)
            }
        }.receiveAsFlow()
    }
}